@@ -136,7 +136,6 @@ wm_new(int argc, char **argv)
      w->img_generic_icon = mb_pixbuf_img_new_from_file(w->pb, GENERIC_ICON);
      if (w->img_generic_icon == NULL)
        {
-	 fprintf(stderr,"matchbox: WARNING: failed to load %s . Disabling icons.\n", GENERIC_ICON); 
 	 w->config->use_icons = 0;
        }
      else misc_scale_wm_app_icon(w);
@@ -1378,8 +1377,12 @@ wm_handle_configure_request (Wm *w, XConfigureRequestEvent *e )
    if (!(value_mask & CWWidth))  req_w = c->width;
    if (!(value_mask & CWHeight)) req_h = c->height;
    
-   if (e->x <= 0 || !(value_mask & CWX)) req_x = c->x;
-   if (e->y <= 0 || !(value_mask & CWY)) req_y = c->y;
+   if ((e->x <= 0 && !(c->flags & CLIENT_TITLE_HIDDEN_FLAG))
+	|| !(value_mask & CWX)) 
+       req_x = c->x;
+   if ((e->y <= 0 && !(c->flags & CLIENT_TITLE_HIDDEN_FLAG))
+       || !(value_mask & CWY)) 
+     req_y = c->y;
 
    dbg("%s() for win %s - have w: %i vs %i, h: %i" 
        "vs %i, x: %i vs %i, y: %i vs %i,\n", 
@@ -1558,7 +1561,43 @@ wm_handle_configure_request (Wm *w, XConfigureRequestEvent *e )
 
 		 if (trans_client)
 		   {
-		     trans_client->height += diff;
+		     if (trans_client->type == MBCLIENT_TYPE_DIALOG)
+		       {
+			 /* do not grow over their initial size
+			    because of IM resizing smaller */
+			 if (trans_client->height + diff 
+			              > trans_client->init_height)
+			   {
+			     trans_client->height = trans_client->init_height;
+			   }
+			 else if (diff < 0)
+			   {
+			     /* Input window has grown larger so push it 
+			      *	up but keep it on screen.
+			     */
+			     int x, y, w, h;
+			     dialog_client_get_coverage(trans_client, 
+							&x, &y, &w, &h);
+
+			     if (c->y < y+h)
+			       {
+				 int amount = (y+h) - c->y;
+				 
+				 /* move upwards */
+				 trans_client->y -= amount;
+				 /* but not over screen area */
+				 if (y-amount < 0)
+				   {
+				     trans_client->height -= amount-y;
+				     trans_client->y += amount-y;
+				   }
+			       }
+			   }
+		       }
+		     else 	/* Application window */
+		       if (trans_client->type != MBCLIENT_TYPE_DESKTOP)
+			 trans_client->height += diff;
+
 		     trans_client->move_resize(trans_client);
 		     trans_client->redraw(trans_client, False);
 		   }
@@ -1991,6 +2030,9 @@ wm_make_new_client(Wm *w, Window win)
 		   dbg("%s() got toolbar atom\n", __func__ );
 		   c = toolbar_client_new(w, win);
 		   if (c == NULL) goto end;
+#ifdef USE_ALT_INPUT_WIN
+		   c->flags &= ~CLIENT_IS_MINIMIZED;
+#endif
 		 }
 	       else if (value[0] == w->atoms[WINDOW_TYPE_DESKTOP])
 		 {
@@ -2005,19 +2047,20 @@ wm_make_new_client(Wm *w, Window win)
 		   if (c == NULL) goto end;
 		   c->flags |= (CLIENT_TITLE_HIDDEN_FLAG|CLIENT_IS_SPLASH_WIN);
 		 }
-	       else if (value[0] == w->atoms[WINDOW_TYPE_DIALOG]
-			|| value[0] == w->atoms[WINDOW_TYPE_MENU])
+	       else if (value[0] == w->atoms[WINDOW_TYPE_DIALOG])
 		 {
 		   dbg("%s() got type dialog atom\n", __func__ );
 		   c = dialog_client_new(w, win, NULL);
 		   if (c == NULL) goto end;
-
-		   /* check to see if we're a 'menu' dialog  */
-		   if (value[0] == w->atoms[WINDOW_TYPE_MENU])
-		     {
-		       dbg("%s() Got menu hint\n", __func__);
-		       c->flags |= CLIENT_IS_MENU_DIALOG;
-		     }
+		 }
+	       else if (value[0] == w->atoms[WINDOW_TYPE_MENU]
+			|| value[0] == w->atoms[_NET_WM_WINDOW_TYPE_POPUP_MENU]
+			|| value[0] == w->atoms[_NET_WM_WINDOW_TYPE_DROPDOWN_MENU])
+		 {
+		   dbg("%s() got type menu atom\n", __func__ );
+		   c = dialog_client_new(w, win, NULL);
+		   if (c == NULL) goto end;
+		   c->flags |= CLIENT_IS_MENU_DIALOG;
 		 }
 	       else if (value[0] == w->atoms[WINDOW_TYPE_MESSAGE] 
 			/* Above to depricate for below */
@@ -2097,6 +2140,12 @@ wm_make_new_client(Wm *w, Window win)
       else if (c->type == MBCLIENT_TYPE_DIALOG) /* already exists, update  */
 	c->trans = t;  		/* TODO: what about other types 
 				         being transient for things ?*/
+
+      /* Make sure above state is inherited if parent has it */
+      if (c->trans != NULL
+	  && c->trans->type == MBCLIENT_TYPE_DIALOG 
+	  && c->trans->flags & CLIENT_HAS_ABOVE_STATE)
+	c->flags |= CLIENT_HAS_ABOVE_STATE;
    }
    
    if (c == NULL) /* Noting else found, default to a main client */
@@ -2128,6 +2177,9 @@ wm_make_new_client(Wm *w, Window win)
    
    c->move_resize(c);          	/* set pos + size */
 
+   /* send new configuration to client - needed */
+   client_deliver_config(c);
+
    /* TODO:
     *
     * Its likely the size we given the new client, is not what it requested. 
@@ -2145,15 +2197,32 @@ wm_make_new_client(Wm *w, Window win)
     * Note, this seems worst on GTK apps.
     */
 
+   XGrabButton(c->wm->dpy, Button1, 0, c->window, True, ButtonPressMask,
+	       GrabModeSync, GrabModeSync, None, None);
+
+   /* Handle an application started iconized */
+   if (c->flags & CLIENT_IS_MINIMIZED
+       && c->type == MBCLIENT_TYPE_APP)
+     {
+       /* Clear the flag now to be safe */
+       c->flags &= ~CLIENT_IS_MINIMIZED;
+
+       c->redraw(c, False);		/* draw the decorations ready */
+       c->iconize (c);
+
+       ewmh_update_lists(w); 
+
+       XUngrabServer(w->dpy);
+       XFlush(w->dpy);
+
+       goto end;
+     }
+
    dbg("%s() showing new client\n", __func__);
 
    c->redraw(c, False);		/* draw the decorations ready */
-
    wm_activate_client(c);       /* Map it into stack, ( will call show()) */
 
-   XGrabButton(c->wm->dpy, Button1, 0, c->window, True, ButtonPressMask,
-	       GrabModeSync, GrabModeSync, None, None);
-
    /* Let window know were all done */
 
    ewmh_state_set(c); 		/* XXX This is likely not needed */
